@@ -15,7 +15,10 @@
 #include "card_imp.h"
 
 #include <winscard.h>
-#include <wintypes.h>
+
+#ifndef OS_WIN32
+# include <wintypes.h>
+#endif
 
 
 
