@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: main.c,v $
+ $RCSfile$
                              -------------------
-    cvs         : $Id: main.c,v 1.2 2005/02/15 10:18:37 aquamaniac Exp $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -14,7 +14,6 @@
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
-#undef BUILDING_LIBCHIPCARD2_DLL
 
 #include <errno.h>
 #include <string.h>
@@ -24,16 +23,16 @@
 #include "global.h"
 #include <gwenhywfar/args.h>
 #include <gwenhywfar/db.h>
-#include <gwenhywfar/nettransportssl.h>
+#include <gwenhywfar/cgui.h>
 
 #define I18N(msg) msg
 
 
-#define PROGRAM_VERSION "1.9"
+#define PROGRAM_VERSION "2.9"
 
 #define k_PRG_VERSION_INFO \
-    "memcard v1.9  (part of libchipcard v"k_CHIPCARD_VERSION_STRING")\n"\
-    "(c) 2003 Martin Preuss<martin@libchipcard.de>\n" \
+    "memcard v2.9  (part of libchipcard v"k_CHIPCARD_VERSION_STRING")\n"\
+    "(c) 2006 Martin Preuss<martin@libchipcard.de>\n" \
     "This program is free software licensed under GPL.\n"\
     "See COPYING for details.\n"
 
@@ -41,18 +40,7 @@
 const GWEN_ARGS prg_args[]={
 {
   GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
-  GWEN_ArgsTypeChar,            /* type */
-  "configfile",                 /* name */
-  0,                            /* minnum */
-  1,                            /* maxnum */
-  "C",                          /* short option */
-  "configfile",                 /* long option */
-  "Configuration file to load", /* short description */
-  "Configuration file to load." /* long description */
-},
-{
-  GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
-  GWEN_ArgsTypeChar,            /* type */
+  GWEN_ArgsType_Char,            /* type */
   "logtype",                    /* name */
   0,                            /* minnum */
   1,                            /* maxnum */
@@ -63,7 +51,7 @@ const GWEN_ARGS prg_args[]={
 },
 {
   GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
-  GWEN_ArgsTypeChar,            /* type */
+  GWEN_ArgsType_Char,            /* type */
   "loglevel",                   /* name */
   0,                            /* minnum */
   1,                            /* maxnum */
@@ -74,7 +62,7 @@ const GWEN_ARGS prg_args[]={
 },
 {
   GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
-  GWEN_ArgsTypeChar,            /* type */
+  GWEN_ArgsType_Char,            /* type */
   "logfile",                    /* name */
   0,                            /* minnum */
   1,                            /* maxnum */
@@ -85,7 +73,7 @@ const GWEN_ARGS prg_args[]={
 },
 {
   0,                            /* flags */
-  GWEN_ArgsTypeInt,             /* type */
+  GWEN_ArgsType_Int,             /* type */
   "verbosity",                  /* name */
   0,                            /* minnum */
   10,                           /* maxnum */
@@ -96,7 +84,7 @@ const GWEN_ARGS prg_args[]={
 },
 {
   0,                            /* flags */
-  GWEN_ArgsTypeInt,             /* type */
+  GWEN_ArgsType_Int,             /* type */
   "verify",                     /* name */
   0,                            /* minnum */
   1,                            /* maxnum */
@@ -107,7 +95,7 @@ const GWEN_ARGS prg_args[]={
 },
 {
   GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
-  GWEN_ArgsTypeInt,             /* type */
+  GWEN_ArgsType_Int,             /* type */
   "offset",                     /* name */
   0,                            /* minnum */
   1,                            /* maxnum */
@@ -118,7 +106,7 @@ const GWEN_ARGS prg_args[]={
 },
 {
   GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
-  GWEN_ArgsTypeInt,             /* type */
+  GWEN_ArgsType_Int,             /* type */
   "size",                       /* name */
   1,                            /* minnum */
   1,                            /* maxnum */
@@ -129,7 +117,7 @@ const GWEN_ARGS prg_args[]={
 },
 {
   GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
-  GWEN_ArgsTypeChar,            /* type */
+  GWEN_ArgsType_Char,            /* type */
   "filename",                   /* name */
   0,                            /* minnum */
   1,                            /* maxnum */
@@ -140,7 +128,7 @@ const GWEN_ARGS prg_args[]={
 },
 {
   GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
-  GWEN_ArgsTypeChar,            /* type */
+  GWEN_ArgsType_Char,            /* type */
   "pin",                        /* name */
   0,                            /* minnum */
   1,                            /* maxnum */
@@ -151,7 +139,7 @@ const GWEN_ARGS prg_args[]={
 },
 {
   GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
-  GWEN_ArgsTypeInt,             /* type */
+  GWEN_ArgsType_Int,             /* type */
   "help",                       /* name */
   0,                            /* minnum */
   0,                            /* maxnum */
@@ -164,9 +152,9 @@ const GWEN_ARGS prg_args[]={
 
 
 void usage(const char *ustr) {
-  fprintf(stderr,"%s%s",
+  fprintf(stdout,"%s%s",
           I18N("MemCard2 - A tool to read/write data from/to a memory chip card\n"
-               "(c) 2004 Martin Preuss<martin@libchipcard.de>\n"
+               "(c) 2006 Martin Preuss<martin@libchipcard.de>\n"
                "This library is free software; you can redistribute it and/or\n"
                "modify it under the terms of the GNU Lesser General Public\n"
                "License as published by the Free Software Foundation; either\n"
@@ -214,7 +202,7 @@ void showError(LC_CARD *card, LC_CLIENT_RESULT res, const char *x) {
   }
 
   fprintf(stderr, "Error in \"%s\": %s\n", x, s);
-  if (res==LC_Client_ResultCmdError) {
+  if (card && res==LC_Client_ResultCmdError) {
     int sw1;
     int sw2;
 
@@ -254,7 +242,7 @@ int memRead(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
   v=GWEN_DB_GetIntValue(dbArgs, "verbosity", 0, 0);
   if (v>1)
     fprintf(stderr, "Connecting to server.\n");
-  res=LC_Client_StartWait(cl, 0, 0);
+  res=LC_Client_Start(cl);
   if (res!=LC_Client_ResultOk) {
     showError(card, res, "StartWait");
     return RETURNVALUE_WORK;
@@ -265,16 +253,28 @@ int memRead(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
   for (i=0;;i++) {
     if (v>0)
       fprintf(stderr, "Waiting for card...\n");
-    card=LC_Client_WaitForNextCard(cl, 2);
-    if (card) {
-      if (v>0)
-        fprintf(stderr, "Found a card.\n");
-
-      s=LC_Card_GetCardType(card);
-      assert(s);
-      if (strcasecmp(s, "memory")==0)
-        break;
+    res=LC_Client_GetNextCard(cl, &card, 20);
+    if (res!=LC_Client_ResultOk) {
+      showError(card, res, "GetNextCard");
+      return RETURNVALUE_WORK;
     }
+    if (v>0)
+      fprintf(stderr, "Found a card.\n");
+
+    s=LC_Card_GetCardType(card);
+    assert(s);
+    if (strcasecmp(s, "memory")==0)
+      break;
+
+    if (v>0)
+      fprintf(stderr, "Not a memory card, releasing.\n");
+    res=LC_Client_ReleaseCard(cl, card);
+    if (res!=LC_Client_ResultOk) {
+      showError(card, res, "ReleaseCard");
+      return RETURNVALUE_WORK;
+    }
+    LC_Card_free(card);
+
     if (i>15) {
       fprintf(stderr, "ERROR: No card found.\n");
       return RETURNVALUE_WORK;
@@ -304,9 +304,10 @@ int memRead(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
   /* stop waiting */
   if (v>1)
     fprintf(stderr, "Telling the server that we need no more cards.\n");
-  res=LC_Client_StopWait(cl);
+  res=LC_Client_Stop(cl);
   if (res!=LC_Client_ResultOk) {
-    showError(card, res, "StopWait");
+    showError(card, res, "Stop");
+    LC_Client_ReleaseCard(cl, card);
     return RETURNVALUE_WORK;
   }
 
@@ -323,6 +324,8 @@ int memRead(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
             I18N("ERROR: Could not open file (%s).\n"),
             strerror(errno));
     LC_Card_Close(card);
+    LC_Client_ReleaseCard(cl, card);
+
     return RETURNVALUE_WORK;
   }
 
@@ -366,6 +369,7 @@ int memRead(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
       if (fname)
         fclose(f);
       LC_Card_Close(card);
+      LC_Client_ReleaseCard(cl, card);
       GWEN_Buffer_free(buf);
       return RETURNVALUE_WORK;
     }
@@ -383,6 +387,7 @@ int memRead(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
       if (fname)
         fclose(f);
       LC_Card_Close(card);
+      LC_Client_ReleaseCard(cl, card);
       GWEN_Buffer_free(buf);
       return RETURNVALUE_WORK;
     }
@@ -426,6 +431,15 @@ int memRead(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
     if (v>1)
       fprintf(stderr, "Card closed.\n");
 
+  if (v>0)
+    fprintf(stderr, "Releasing card.\n");
+  res=LC_Client_ReleaseCard(cl, card);
+  if (res!=LC_Client_ResultOk) {
+    showError(card, res, "ReleaseCard");
+    return RETURNVALUE_WORK;
+  }
+  LC_Card_free(card);
+
   /* finished */
   if (v>1)
     fprintf(stderr, "Finished.\n");
@@ -471,7 +485,7 @@ int memWrite(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
 
   if (v>1)
     fprintf(stderr, "Connecting to server.\n");
-  res=LC_Client_StartWait(cl, 0, 0);
+  res=LC_Client_Start(cl);
   if (res!=LC_Client_ResultOk) {
     showError(card, res, "StartWait");
     if (fname)
@@ -484,16 +498,32 @@ int memWrite(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
   for (i=0;;i++) {
     if (v>0)
       fprintf(stderr, "Waiting for card...\n");
-    card=LC_Client_WaitForNextCard(cl, 2);
-    if (card) {
-      if (v>0)
-        fprintf(stderr, "Found a card.\n");
-
-      s=LC_Card_GetCardType(card);
-      assert(s);
-      if (strcasecmp(s, "memory")==0)
-        break;
+    res=LC_Client_GetNextCard(cl, &card, 20);
+    if (res!=LC_Client_ResultOk) {
+      showError(card, res, "GetNextCard");
+      if (fname)
+        fclose(f);
+      return RETURNVALUE_WORK;
     }
+    if (v>0)
+      fprintf(stderr, "Found a card.\n");
+
+    s=LC_Card_GetCardType(card);
+    assert(s);
+    if (strcasecmp(s, "memory")==0)
+      break;
+
+    if (v>0)
+      fprintf(stderr, "Not a memory card, releasing.\n");
+    res=LC_Client_ReleaseCard(cl, card);
+    if (res!=LC_Client_ResultOk) {
+      showError(card, res, "ReleaseCard");
+      if (fname)
+        fclose(f);
+      return RETURNVALUE_WORK;
+    }
+    LC_Card_free(card);
+
     if (i>15) {
       fprintf(stderr, "ERROR: No card found.\n");
       if (fname)
@@ -508,6 +538,7 @@ int memWrite(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
     fprintf(stderr, "Could not extend card as memory card\n");
     if (fname)
       fclose(f);
+    LC_Client_ReleaseCard(cl, card);
     return RETURNVALUE_WORK;
   }
 
@@ -521,6 +552,7 @@ int memWrite(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
             res);
     if (fname)
       fclose(f);
+    LC_Client_ReleaseCard(cl, card);
     return RETURNVALUE_WORK;
   }
   if (v>0)
@@ -529,11 +561,12 @@ int memWrite(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
   /* stop waiting */
   if (v>1)
     fprintf(stderr, "Telling the server that we need no more cards.\n");
-  res=LC_Client_StopWait(cl);
+  res=LC_Client_Stop(cl);
   if (res!=LC_Client_ResultOk) {
-    showError(card, res, "StopWait");
+    showError(card, res, "Stop");
     if (fname)
       fclose(f);
+    LC_Client_ReleaseCard(cl, card);
     return RETURNVALUE_WORK;
   }
 
@@ -567,6 +600,7 @@ int memWrite(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
       if (fname)
         fclose(f);
       LC_Card_Close(card);
+      LC_Client_ReleaseCard(cl, card);
       return RETURNVALUE_WORK;
     }
 
@@ -592,6 +626,7 @@ int memWrite(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
       if (fname)
         fclose(f);
       LC_Card_Close(card);
+      LC_Client_ReleaseCard(cl, card);
       return RETURNVALUE_WORK;
     }
 
@@ -613,6 +648,7 @@ int memWrite(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
         if (fname)
           fclose(f);
         LC_Card_Close(card);
+        LC_Client_ReleaseCard(cl, card);
         GWEN_Buffer_free(tbuf);
         return RETURNVALUE_WORK;
       }
@@ -624,6 +660,7 @@ int memWrite(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
         if (fname)
           fclose(f);
         LC_Card_Close(card);
+        LC_Client_ReleaseCard(cl, card);
         GWEN_Buffer_free(tbuf);
         return RETURNVALUE_WORK;
       }
@@ -653,6 +690,7 @@ int memWrite(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
               I18N("ERROR: Could not close file (%s).\n"),
               strerror(errno));
       LC_Card_Close(card);
+      LC_Client_ReleaseCard(cl, card);
       return RETURNVALUE_WORK;
     }
   }
@@ -669,6 +707,15 @@ int memWrite(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
     if (v>1)
       fprintf(stderr, "Card closed.\n");
 
+  res=LC_Client_ReleaseCard(cl, card);
+  if (res!=LC_Client_ResultOk) {
+    showError(card, res, "ReleaseCard");
+    return RETURNVALUE_WORK;
+  }
+  else
+    if (v>1)
+      fprintf(stderr, "Card released.\n");
+
   /* finished */
   if (v>1)
     fprintf(stderr, "Finished.\n");
@@ -677,13 +724,6 @@ int memWrite(LC_CLIENT *cl, GWEN_DB_NODE *dbArgs){
 
 
 
-GWEN_NETTRANSPORTSSL_ASKADDCERT_RESULT _askAddCert(GWEN_NETTRANSPORT *tr,
-                                                   GWEN_DB_NODE *cert){
-  return GWEN_NetTransportSSL_AskAddCertResultTmp;
-}
-
-
-
 int main(int argc, char **argv) {
   int rv;
   GWEN_DB_NODE *db;
@@ -691,19 +731,22 @@ int main(int argc, char **argv) {
   LC_CLIENT *cl;
   GWEN_LOGGER_LOGTYPE logType;
   GWEN_LOGGER_LEVEL logLevel;
+  LC_CLIENT_RESULT res;
+  GWEN_GUI *gui;
 
-  GWEN_NetTransportSSL_SetAskAddCertFn(_askAddCert);
+  gui=GWEN_Gui_CGui_new();
+  GWEN_Gui_SetGui(gui);
 
   db=GWEN_DB_Group_new("arguments");
   rv=GWEN_Args_Check(argc, argv, 1,
                      GWEN_ARGS_MODE_ALLOW_FREEPARAM,
                      prg_args,
                      db);
-  if (rv==-2) {
+  if (rv==GWEN_ARGS_RESULT_HELP) {
     GWEN_BUFFER *ubuf;
 
     ubuf=GWEN_Buffer_new(0, 256, 0, 1);
-    if (GWEN_Args_Usage(prg_args, ubuf, GWEN_ArgsOutTypeTXT)) {
+    if (GWEN_Args_Usage(prg_args, ubuf, GWEN_ArgsOutType_Txt)) {
       fprintf(stderr, "Could not generate usage string.\n");
       GWEN_Buffer_free(ubuf);
       return RETURNVALUE_PARAM;
@@ -720,21 +763,21 @@ int main(int argc, char **argv) {
   /* setup logging */
   s=GWEN_DB_GetCharValue(db, "loglevel", 0, "warning");
   logLevel=GWEN_Logger_Name2Level(s);
-  if (logLevel==GWEN_LoggerLevelUnknown) {
+  if (logLevel==GWEN_LoggerLevel_Unknown) {
     fprintf(stderr, "ERROR: Unknown log level (%s)\n", s);
     return RETURNVALUE_PARAM;
   }
   s=GWEN_DB_GetCharValue(db, "logtype", 0, "console");
   logType=GWEN_Logger_Name2Logtype(s);
-  if (logType==GWEN_LoggerTypeUnknown) {
+  if (logType==GWEN_LoggerType_Unknown) {
     fprintf(stderr, "ERROR: Unknown log type (%s)\n", s);
     return RETURNVALUE_PARAM;
   }
   rv=GWEN_Logger_Open(LC_LOGDOMAIN,
-                      "memcard2",
-		      GWEN_DB_GetCharValue(db, "logfile", 0, "memcard2.log"),
+                      "memcard3",
+		      GWEN_DB_GetCharValue(db, "logfile", 0, "memcard3.log"),
 		      logType,
-		      GWEN_LoggerFacilityUser);
+		      GWEN_LoggerFacility_User);
   if (rv) {
     fprintf(stderr, "ERROR: Could not setup logging (%d).\n", rv);
     return RETURNVALUE_SETUP;
@@ -749,13 +792,10 @@ int main(int argc, char **argv) {
     return RETURNVALUE_PARAM;
   }
 
-  cl=LC_Client_new("memcard2", PROGRAM_VERSION, 0);
-  if (LC_Client_ReadConfigFile(cl,
-                               GWEN_DB_GetCharValue(db, "configfile",
-                                                    0, 0))) {
-    fprintf(stderr, "Error reading configuration.\n");
-    LC_Client_free(cl);
-    GWEN_DB_Group_free(db);
+  cl=LC_Client_new("memcard", PROGRAM_VERSION);
+  res=LC_Client_Init(cl);
+  if (res!=LC_Client_ResultOk) {
+    showError(0, res, "Init");
     return RETURNVALUE_SETUP;
   }
 
@@ -771,6 +811,11 @@ int main(int argc, char **argv) {
     rv=RETURNVALUE_PARAM;
   }
 
+  res=LC_Client_Fini(cl);
+  if (res!=LC_Client_ResultOk) {
+    showError(0, res, "Init");
+  }
+
   LC_Client_free(cl);
   GWEN_DB_Group_free(db);
   return 0;
